%initializing the program
clear all
%close all
clc

%start the timer
tic

%The plant parameters
% A = [1 1; 0 1];
% B = [0.5; 1];

A = [2.2 -1.2; 1.0 0.0];
B = [0.8; 0.4];


%Incremental covariance of the continuous-time brownian motion
%standard deviation of signal
R = [1 0.2; 0.2 1];

%Number of time steps after which automatic transmission happens 
%(max 4 for this program)
T = 4;

%mean
mu = zeros(1,8);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Multi-dimensional covariance matrix 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cov_mat = zeros(2*T,2*T);
%
cov_mat(1:2,1:2) = A*R*(A^1)' + R;
cov_mat(1:2,3:4) = A*R*(A^2)' + R*(A^1)';
cov_mat(1:2,5:6) = A*R*(A^3)' + R*(A^2)';
cov_mat(1:2,7:8) = A*R*(A^4)' + R*(A^3)';

cov_mat(3:4,1:2) = cov_mat(1:2,3:4)';
cov_mat(3:4,3:4) = (A^2)*R*(A^2)' + A*R*(A^1)' + R;
cov_mat(3:4,5:6) = (A^2)*R*(A^3)' + A*R*(A^2)' + R*(A^1)';
cov_mat(3:4,7:8) = (A^2)*R*(A^4)' + A*R*(A^3)' + R*(A^2)';

cov_mat(5:6,1:2) = cov_mat(1:2,5:6)';
cov_mat(5:6,3:4) = cov_mat(3:4,5:6)';
cov_mat(5:6,5:6) = (A^3)*R*(A^3)' + (A^2)*R*(A^2)' + A*R*(A^1)' + R;
cov_mat(5:6,7:8) = (A^3)*R*(A^4)' + (A^2)*R*(A^3)' + A*R*(A^2)' + R*(A^1)';

cov_mat(7:8,1:2) = cov_mat(1:2,7:8)';
cov_mat(7:8,3:4) = cov_mat(3:4,7:8)';
cov_mat(7:8,5:6) = cov_mat(5:6,7:8)';
cov_mat(7:8,7:8) = (A^4)*R*(A^4)' + (A^3)*R*(A^3)' + (A^2)*R*(A^2)' + A*R*(A^1)' + R;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Threshold span
eps = 0.2:0.2:4.0;
%
% eps = 1.8:0.2:2.0;

% iterations
k1 = 1;
k2 = 2;
k3 = 3;
k4 = 4;
      
for iter = 1:length(eps),

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    xl = [eps(iter) eps(iter)];
    xu = [inf       inf      ];
    p(k1,1) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [-inf       eps(iter)];
    xu = [-eps(iter) inf      ];
    p(k1,2) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [-inf       -inf      ];
    xu = [-eps(iter) -eps(iter)];
    p(k1,3) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [eps(iter) -inf      ];
    xu = [inf       -eps(iter)];
    p(k1,4) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [-eps(iter) eps(iter)];
    xu = [ eps(iter) inf      ];
    p(k1,5) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [-inf       -eps(iter)];
    xu = [-eps(iter)  eps(iter)];
    p(k1,6) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [-eps(iter) -inf      ];
    xu = [ eps(iter) -eps(iter)];
    p(k1,7) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    xl = [eps(iter) -eps(iter)];
    xu = [inf        eps(iter)];
    p(k1,8) = mvncdf(xl,xu,mu(1:2),cov_mat(1:2,1:2));
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    xl = [-eps(iter)*ones(1,2), eps(iter) eps(iter)];
    xu = [ eps(iter)*ones(1,2), inf       inf      ];
    p(k2,1) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), -inf       eps(iter)];
    xu = [ eps(iter)*ones(1,2), -eps(iter) inf      ];
    p(k2,2) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), -inf       -inf      ];
    xu = [ eps(iter)*ones(1,2), -eps(iter) -eps(iter)];
    p(k2,3) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), eps(iter) -inf      ];
    xu = [ eps(iter)*ones(1,2), inf       -eps(iter)];
    p(k2,4) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), -eps(iter) eps(iter)];
    xu = [ eps(iter)*ones(1,2),  eps(iter) inf      ];
    p(k2,5) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), -inf       -eps(iter)];
    xu = [ eps(iter)*ones(1,2), -eps(iter)  eps(iter)];
    p(k2,6) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), -eps(iter) -inf      ];
    xu = [ eps(iter)*ones(1,2),  eps(iter) -eps(iter)];
    p(k2,7) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    xl = [-eps(iter)*ones(1,2), eps(iter) -eps(iter)];
    xu = [ eps(iter)*ones(1,2), inf        eps(iter)];
    p(k2,8) = mvncdf(xl,xu,mu(1:4),cov_mat(1:4,1:4));
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    xl = [-eps(iter)*ones(1,4), eps(iter) eps(iter)];
    xu = [ eps(iter)*ones(1,4), inf       inf      ];
    p(k3,1) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), -inf       eps(iter)];
    xu = [ eps(iter)*ones(1,4), -eps(iter) inf      ];
    p(k3,2) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), -inf       -inf      ];
    xu = [ eps(iter)*ones(1,4), -eps(iter) -eps(iter)];
    p(k3,3) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), eps(iter) -inf      ];
    xu = [ eps(iter)*ones(1,4), inf       -eps(iter)];
    p(k3,4) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), -eps(iter) eps(iter)];
    xu = [ eps(iter)*ones(1,4),  eps(iter) inf      ];
    p(k3,5) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), -inf       -eps(iter)];
    xu = [ eps(iter)*ones(1,4), -eps(iter)  eps(iter)];
    p(k3,6) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), -eps(iter) -inf      ];
    xu = [ eps(iter)*ones(1,4),  eps(iter) -eps(iter)];
    p(k3,7) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    xl = [-eps(iter)*ones(1,4), eps(iter) -eps(iter)];
    xu = [ eps(iter)*ones(1,4), inf        eps(iter)];
    p(k3,8) = mvncdf(xl,xu,mu(1:6),cov_mat(1:6,1:6));
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    %
    xl = [-eps(iter)*ones(1,6), eps(iter) eps(iter)];
    xu = [ eps(iter)*ones(1,6), inf       inf      ];
    p(k4,1) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), -inf       eps(iter)];
    xu = [ eps(iter)*ones(1,6), -eps(iter) inf      ];
    p(k4,2) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), -inf       -inf      ];
    xu = [ eps(iter)*ones(1,6), -eps(iter) -eps(iter)];
    p(k4,3) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), eps(iter) -inf      ];
    xu = [ eps(iter)*ones(1,6), inf       -eps(iter)];
    p(k4,4) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), -eps(iter) eps(iter)];
    xu = [ eps(iter)*ones(1,6),  eps(iter) inf      ];
    p(k4,5) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), -inf       -eps(iter)];
    xu = [ eps(iter)*ones(1,6), -eps(iter)  eps(iter)];
    p(k4,6) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), -eps(iter) -inf      ];
    xu = [ eps(iter)*ones(1,6),  eps(iter) -eps(iter)];
    p(k4,7) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    xl = [-eps(iter)*ones(1,6), eps(iter) -eps(iter)];
    xu = [ eps(iter)*ones(1,6), inf        eps(iter)];
    p(k4,8) = mvncdf(xl,xu,mu(1:8),cov_mat(1:8,1:8));
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %iteration
    k1 = k1 + 4;
    k2 = k2 + 4;
    k3 = k3 + 4;
    k4 = k4 + 4;    
    %
end

%stop the timer
toc

% mu = [0 0];
% 
% xl = [-eps -eps];
% xu = [ eps  eps];
% 
% q = mvncdf(xl,xu,mu,cov_mat(1:2,1:2))

